@@ -8,7 +8,7 @@ package Aniki {
     use Aniki::Schema;
     use Aniki::QueryBuilder;
 
-    our $VERSION = '0.03_01';
+    our $VERSION = '0.04_01';
 
     use SQL::Maker::SQLType qw/sql_type/;
     use DBIx::Sunny;
@@ -280,12 +280,13 @@ package Aniki {
 
         local $self->{suppress_row_objects} = 1 if $opt->{suppress_row_objects};
 
-        my @columns = ('*');
         my $table = $self->schema->get_table($table_name);
-        if ($table) {
-            $where   = $self->_bind_sql_type_to_args($table, $where);
-            @columns = map { $_->name } $table->get_fields();
-        }
+
+        my @columns = exists $opt->{columns} ? @{ $opt->{columns} }
+                    : defined $table ? map { $_->name } $table->get_fields()
+                    : ('*');
+
+        $where = $self->_bind_sql_type_to_args($table, $where) if defined $table;
 
         local $Carp::CarpLevel = $Carp::CarpLevel + 1;
         my ($sql, @bind) = $self->query_builder->select($table_name, \@columns, $where, $opt);
@@ -366,7 +367,8 @@ package Aniki {
         push @rows => {%row} while $sth->fetch;
         $sth->finish;
 
-        return $self->result_class->new(
+        my $result_class = $self->guess_result_class($table_name);
+        return $result_class->new(
             table_name => $table_name,
             handler    => $self,
             row_datas  => \@rows,
@@ -431,6 +433,18 @@ package Aniki {
         };
     }
 
+    sub guess_result_class {
+        my ($self, $table_name) = @_;
+        my $result_class = sprintf '%s::%s', $self->result_class, camelize($table_name);
+        return try {
+            Module::Load::load($result_class);
+            return $result_class;
+        } catch {
+            die $_ unless /\A\QCan't locate/imo;
+            return $self->result_class;
+        };
+    }
+
     sub new_row_from_hashref {
         my ($self, $table_name, $row_data) = @_;
         return $row_data if $self->suppress_row_objects;
@@ -447,7 +461,8 @@ package Aniki {
         my ($self, $table_name, $row_datas) = @_;
         return $row_datas if $self->suppress_row_objects;
 
-        return $self->result_class->new(
+        my $result_class = $self->guess_result_class($table_name);
+        return $result_class->new(
             table_name => $table_name,
             handler    => $self,
             row_datas  => $row_datas,
